@@ -267,12 +267,11 @@ def main(input_args):
         entrypoint = input_args.index('--entrypoint')
         del input_args[entrypoint]
         input_args.insert(0, input_args.pop(entrypoint))
-        return subprocess.call(input_args)
+        return subprocess.run(input_args, capture_output=True, check=True)
     except ValueError:
         # There is no entrypoint override, no worries
         pass
 
-
     parser = argparse.ArgumentParser(
         description=_HELP,
     )
